/**
* @package Enemies
*/
/****************************************************
 *	Part of Shadow World project					*
 *													*
 *	Original name : CEnemy.h						*
 *													*
 *	Header for CEnemy.cpp							*
 *													*
 *													*
 *	May 2003, LeadFiner								*
 *	LeadFiner@wanadoo.fr							*
 ****************************************************/

#if !defined CENEMY
#define CENEMY

#include "Types2.h"
#include "CPerso.h"

/********************************************************************************
 *																				*
 *						Data Declarations and Defines							*
 *																				*
 ********************************************************************************/

#define STATE_CHOOSE_DEST	1
#define STATE_GO_DEST		2

#define LIFE_STATE_ALIVE	0
#define LIFE_STATE_DAMAGE	1
#define LIFE_STATE_KILL		2
#define LIFE_STATE_DIED		3

#define DEFAUT_SPR_EXTRA_ANI_D			128
#define DEFAUT_EXTRA_ANI_SPEED_D		2
#define DEFAUT_NBR_SPR_EXTRA_ANI_D		8*DEFAUT_EXTRA_ANI_SPEED_D
#define DEFAUT_SPR_EXTRA_ANI_K			136
#define DEFAUT_EXTRA_ANI_SPEED_K		2
#define DEFAUT_NBR_SPR_EXTRA_ANI_K		8*DEFAUT_EXTRA_ANI_SPEED_K

#define DEFAULT_ENEMY_ANI_LEN			2

/********************************************************************************
 *																				*
 *						Class Declarations										*
 *																				*
 ********************************************************************************/
/**
* Base class for all enemies
*/
class CEnemy : public CPerso
{

public:
	/**
	* Standard constructor
	* @param damage set the damage of the enemy
	* @param life set the life of the enemy
	* @see CPerso#CPerso()
	*/
	CEnemy(uchar objID, short initX, short initY
		, char margeBottom
		, char margeLeft
		, char margeTop
		, char margeRight
		, char damage
		, char life
		, uchar sprBottom
		, uchar sprLeft
		, uchar sprTop
		, uchar sprRight
		, char aniLen=DEFAULT_ENEMY_ANI_LEN
		, char aniSpeed=DEFAULT_ANI_SPEED);

public:
	/**
	* State of automat in AutoMove()
	*/
	uchar m_stateMove;
	/**
	* Number of step before the direction change
	*/
	uchar m_nbrStep;
	/**
	* Current direction of AutoMove()
	*/
	uchar m_goDir;

	//caracteristics
	/**
	* Damage of the enemy
	*/
	char m_damage;
	//extra animations
	/**
	* State of the enemy. Can be one of this values :
	* <p>
	* LIFE_STATE_ALIVE, LIFE_STATE_DAMAGE, LIFE_STATE_KILL, LIFE_STATE_DIED
	* </p>
	*/
	uchar m_lifeState;
	/**
	* Timer for animation when enemy is damaged or killed
	*/
	uchar m_timerExtraAni;

	//for moving
	/**
	* Configure the maximum of steps for AutoMove()
	*/
	uchar m_stepRnd;
	/**
	* Configure the minimum of steps for AutoMove()
	*/
	uchar m_stepOffset;
	/**
	* Number of pixels for one step
	*/
	uchar m_stepSpeed;
	/**
	* Configure the maximum of directions
	* <p>Default 4</p>
	* <p>>4 for freezing moves</p>
	*/
	uchar m_dirRnd;		//default 4 directions (>4 for step with no move)

public:
	/**
	* Virtual function for implementation in sub-class.
	* <p>By default:</p>
	* <p>- choose number of steps</p>
	* <p>- choose one direction</p>
	* <p>- move the enemy while step<nbrstepmax</p>
	*/
	virtual BOOL AutoMove();
	/**
	* Activate killed animation and set m_lifeState to LIFE_STATE_KILL
	* When animation is finished set m_lifeState to LIFE_STATE_DIED
	* @return TRUE if enemy can move, else FALSE
	*/
	virtual void SetKilled();
	/**
	* Activate damaged animation and set m_lifeState to LIFE_STATE_DAMAGE
	*/
	virtual void SetDamaged();
	/**
	* m_life+=inc
	* <p>For damage use inc<0</p>
	* @return m_life
	*/
	virtual char IncLife(char inc);
	/**
	* Draw sprite for current direction
	*/
	void Draw();

	/**
	* Call CBaseSpr::SetEnable()
	* and set m_timerEnable to 0
	* @see CBaseSpr#SetEnable()
	*/
	inline void SetEnable();

	/**
	* Incremente m_timerEnable
	* @return TRUE if m_timerEnable>TIME_ENABLE
	* @see CSprite
	*/
	BOOL CountDisable();
};

#endif