/****************************************************
 *	Original name : CHero.cpp						*
 *													*
 *	Class for using hero.							*
 *													*
 *													*
 *	June 2003, LeadFiner							*
 ****************************************************/
#include "CHero.h"
#include "Data.h"
#include "LoadFile.h"
#include "Draw.h"
#include "Map.h"
#include "CLinkedS.h"
#include "SW.h"
#include "CEnemy.h"
#include "CItem.h"

/********************************************************************************
 *																				*
 *						Class constructors										*
 *																				*
 ********************************************************************************/

CHero::CHero(short initX, short initY)
	:CPerso(OBJ_ID_HERO, initX, initY
	,HERO_MARGE_BOTTOM
	,HERO_MARGE_LEFT
	,HERO_MARGE_TOP
	,HERO_MARGE_RIGHT
	,HERO_SPR_ID_BOTTOM
	,HERO_SPR_ID_LEFT
	,HERO_SPR_ID_TOP
	,HERO_SPR_ID_RIGHT
	, 0	//life is set in InitGame or when player load game
	,HERO_ANI_LEN)	
{
	m_maxLife=12;
	m_timerGhost=TIME_GHOST;
	m_ghost=TRUE;

	m_weapon1=NULL;

	m_lastProp=P_TERRAN;
}

/********************************************************************************
 *																				*
 *						Class methodes											*
 *																				*
 ********************************************************************************/

BOOL CHero::MoveTo(uchar tDir)
{	
	m_aniLen=HERO_ANI_LEN;

	BOOL returnVal=CPerso::MoveTo(tDir);
	if(returnVal) TestAction();

	//save this good pos
	m_safePosX=m_posX;
	m_safePosY=m_posY;

	hero.m_haveMoved=TRUE;

	return returnVal;
}

BOOL CHero::CanMove()
{
	switch(PropIs())
	{
	case P_STOP:		//can go on water
		return FALSE;
	}

	//default
	return TRUE;
}

char CHero::IncLife(char inc)
{
	if(m_ghost && inc<0) return m_life;

	if(inc>0)
	{
		if(m_life<=(m_maxLife-inc))
			m_life+=inc;
		else m_life=m_maxLife;
	}
	else m_life+=inc;
	
	if(m_life<0) m_life=0;

	return m_life;
}

void CHero::SetGhost()
{
	if(m_ghost) return;


	m_ghost=TRUE;
	m_timerGhost=TIME_GHOST;
}

void CHero::Draw()
{
	if(m_timerGhost%2==0) CPerso::Draw();

	if(m_ghost)
	{
		m_timerGhost--;
		if(m_timerGhost<=0) 
		{
			m_ghost=FALSE;
			TestAction();
		}
	}
	
	if(m_weapon1!=NULL && m_weapon1->IsEnable()) m_weapon1->Draw();
	//if(m_weapon2!=NULL && m_weapon2->IsEnable()) m_weapon2->Draw();
}

/********************************************************************************
 *																				*
 *						Hero functions											*
 *																				*
 ********************************************************************************/

BOOL Pay(uchar value)
{	
	CNode* curNode;

	//search shiny stones
	for(curNode=hero.m_listItems.m_head; curNode!=NULL && curNode->m_sprite->m_objectID!=OBJECT_SHINYS; curNode=curNode->m_next);

	//no shiny stones
	if(curNode==NULL)
	{
		DisplayTextWindow(DISP_WND_BOTTOM,LoadText(TEXT_NOT_SHINES));
		return FALSE;
	}

	//not enough shiny stones
	if(((CItem*)curNode->m_sprite)->m_extra<value)
	{
		DisplayTextWindow(DISP_WND_BOTTOM,LoadText(TEXT_NOT_SHINES));
		return FALSE;
	}

	((CItem*)curNode->m_sprite)->m_extra-=value;
	DisplayTextWindow(DISP_WND_BOTTOM,LoadText(TEXT_THANK_YOU));

	WipeInvHero();

	return TRUE;
}

void FlyTo(short posX, short posY, char scanSquare)
{
	char dirX=(posX>hero.m_posX) ? 1 : -1;
	char dirY=(posY>hero.m_posY) ? 1 : -1;
	short keepPosX;
	short keepPosY;
	
	while(hero.m_posX!=posX || hero.m_posY!=posY)
	{
		if(hero.m_posX==posX) dirX=0;
		if(hero.m_posY==posY) dirY=0;


		keepPosX=hero.m_posX;
		keepPosY=hero.m_posY;

		hero.SetPos(hero.m_posX+dirX,hero.m_posY+dirY);

		if((scanSquare & SCAN_STOP)==SCAN_STOP && (hero.PropIs()==P_STOP || hero.PropIs()==P_TEXT))
		{
			hero.SetPos(keepPosX,keepPosY);
			posX=keepPosX;
			posY=keepPosY;
		}

		if((scanSquare & SCAN_WATER)==SCAN_WATER && hero.PropIs()==P_WATER)
		{
			hero.m_safePosX=keepPosX;
			hero.m_safePosY=keepPosY;

			posX=hero.m_posX;
			posY=hero.m_posY;
		}

		if(!hero.IsInMap())
		{
			hero.m_safePosX=keepPosX;
			hero.m_safePosY=keepPosY;

			posX=hero.m_posX;
			posY=hero.m_posY;
		}

		DrawScene();
		SwapPage();
	}

	TestAction();
}

void TestAction()
{
	uchar curProp=hero.PropIs();

	switch(curProp) 
	{
	case P_TERRAN:
		hero.m_lastProp=curProp;
		return;
	case P_WATER:case P_KILLER12:
		if(curProp==P_WATER)
		{
			FlyTo(hero.m_safePosX, hero.m_safePosY, SCAN_NOTHING);
			hero.IncLife(-1);
		}
		else 
		{
			hero.IncLife(-12);
		}

		hero.SetGhost();
		
		hero.m_lastProp=curProp;
		return;
	}


	// TEXT ///////////////////////////////////////////////
	if (curProp>=P_TEXT && curProp<P_TEXT+MAP_TEXT_MAX)
	{
		LoadText(GetTxtID(curProp%P_TEXT));
		DisplayTextWindow(DISP_WND_TOP,curText);
		FlyTo(hero.m_safePosX, hero.m_safePosY, SCAN_NOTHING);

		hero.m_lastProp=curProp;
		return;
	}

	// JUMP ///////////////////////////////////////////////
	if (curProp>=P_JUMP && curProp<P_JUMP+MAP_JUMP_MAX)
	{
		StructJump* tmpJmp;

		tmpJmp=GetJump(curProp%P_JUMP);	//keepProp%P_JUMP -> ID of Jump

		ushort keepCase=tmpJmp->CaseNbr;

		if(tmpJmp->MapID!=GetMapID())
		{
			LoadMap(tmpJmp->MapID);

			hero.SetPos(CaseToPosPxX(keepCase),CaseToPosPxY(keepCase));

			DrawNextMapFX();
			TestAction();
		}
		else FlyTo(CaseToPosPxX(keepCase),CaseToPosPxY(keepCase), SCAN_NOTHING);
		
		hero.m_lastProp=curProp;
		return;
	}

	if(hero.m_lastProp==curProp) return; //we must exit the square for re-test it

	switch(curProp)
	{
		// RESCAN ///////////////////////////////////////////////
	case OBJECT_RESCAN:	
		LoadMap(GetMapID());
		
		hero.m_lastProp=curProp;
		return;

		// KILL ALL ///////////////////////////////////////////////
	case OBJECT_KILLALL:
		KillAllInMap(GetMapID());
		
		hero.m_lastProp=curProp;
		return;
		// SAVE GAME ///////////////////////////////////////////////
	case OBJECT_SAVE:
		if(DisplayTextWindow(DISP_WND_TOP,LoadText(TEXT_DO_SAVE))==1)
		{
			char slot=DisplayTextWindow(DISP_WND_BOTTOM,LoadText(TEXT_CHOS_SAVE));

			if(slot>=1 && slot<=3)
			{
				hero.MoveTo(hero.NotDirection());
				if(SaveGame(slot)) DisplayTextWindow(DISP_WND_TOP,LoadText(TEXT_OK_SAVED));
				else DisplayTextWindow(DISP_WND_TOP,LoadText(TEXT_NOT_SAVED));
			}
		}
		
		hero.m_lastProp=curProp;
		return;
	case OBJECT_E_RELOAD:
		SetMapEnemyIsScan(GetMapID(),FALSE);
		
		hero.m_lastProp=curProp;
		return;
	case OBJECT_SHIFT_P:
		LoadMap(GetMapID()+1);
		
		hero.m_lastProp=curProp;
		return;
	case OBJECT_SHIFT_M:
		LoadMap(GetMapID()-1);
		
		hero.m_lastProp=curProp;
		return;
	case P_EQUALIZER:
		CNode* curNode;

		//scan items
		for(curNode=hero.m_listItems.m_head; curNode!=NULL; curNode=curNode->m_next)
		{
			switch(curNode->m_sprite->m_objectID)
			{
			case OBJECT_SHINYS:
				if(((CItem*)curNode->m_sprite)->m_extra>30) ((CItem*)curNode->m_sprite)->m_extra=30;
				break;
			case OBJECT_HEART:
				if(((CItem*)curNode->m_sprite)->m_extra>3) ((CItem*)curNode->m_sprite)->m_extra=3;
				break;
			}
		}
		//scan weapons
		for(curNode=hero.m_listWeapons.m_head; curNode!=NULL; curNode=curNode->m_next)
		{
			switch(curNode->m_sprite->m_objectID)
			{
			case OBJECT_BOMB:
				if(((CItem*)curNode->m_sprite)->m_extra>10) ((CItem*)curNode->m_sprite)->m_extra=10;
				break;
			case OBJECT_ARROW:
				if(((CItem*)curNode->m_sprite)->m_extra>12) ((CItem*)curNode->m_sprite)->m_extra=12;
				break;
			}
		}
		return;
	case P_KILLER:
		hero.m_life=0;
		return;
	}
}			


void WipeInvHero()
{
	CNode* curNode=hero.m_listItems.m_head;
	
	//delete empty items
	while(curNode!=NULL)
	{
		if(((CItem*)curNode->m_sprite)->m_extra<=0)
		{
				curNode=hero.m_listItems.DeleteSprite(curNode);
		}
		else curNode=curNode->m_next;
	}

	//delete empty weapons
	curNode=hero.m_listWeapons.m_head;
	while(curNode!=NULL)
	{
		if(((CWeapon*)curNode->m_sprite)->m_extra<=0)
		{
				curNode=hero.m_listWeapons.DeleteSprite(curNode);
		}
		else curNode=curNode->m_next;
	}
}