/****************************************************
 *	Original name : CLinkedS.cpp					*
 *													*
 *	Class for using multiple sprites.				*
 *													*
 *													*
 *	June 2003, LeadFiner							*
 ****************************************************/

#include "CLinkedS.h"
#include "System2.h"
#include "Draw.h"
/********************************************************************************
 *																				*
 *						Class CNode constructors								*
 *																				*
 ********************************************************************************/

CNode::CNode()
{
	m_next=NULL;
	m_sprite=NULL;
}

/********************************************************************************
 *																				*
 *						Class CLinkedSprites constructors						*
 *																				*
 ********************************************************************************/

CLinkedSprites::CLinkedSprites()
{
	m_head=NULL;
	m_tail=NULL;
}

/********************************************************************************
 *																				*
 *						Class CLinkedSprites methodes							*
 *																				*
 ********************************************************************************/

CBaseSpr* CLinkedSprites::AddSprite(CBaseSpr *ptSpr)
{
	if(ptSpr==NULL) return NULL;

	CNode* newNode=new CNode();

	if(newNode==NULL) //out of memory ???
	{
		delete ptSpr;
		DeleteSprite(m_head);
		return NULL;		
	}

	newNode->m_sprite=ptSpr;

	if(m_head==NULL)
	{
		m_head=newNode;
	}
	else
	{
		m_tail->m_next=newNode;
	}

	m_tail=newNode;

	return ptSpr;
}

BOOL CLinkedSprites::IsEmpty()
{
	return m_head==NULL;
}

void CLinkedSprites::DeleteAll()
{
	while(m_head!=NULL) DeleteSprite(m_head);
}

CNode* CLinkedSprites::DeleteSprite(CNode *ptNode)
{
	if(IsEmpty()) return NULL;		//nothing to delete...

	CNode* node=m_head;
	CNode* nodeToDel=m_head;

	//sprite is the head of list
	if(nodeToDel==ptNode)
	{
		m_head=nodeToDel->m_next;
		
		if(nodeToDel==m_tail)
		{
			m_tail=NULL;
			m_head->m_next=NULL;
		}
		else
		{
			m_head->m_next=nodeToDel->m_next->m_next;
		}

		delete nodeToDel->m_sprite;
		delete nodeToDel;

		return m_head;
	}

	//sprite isn't the head of list
	for(;node->m_next!=NULL; node=node->m_next)
	{
		nodeToDel=node->m_next;

		if(nodeToDel==ptNode) break;
	}

	if(nodeToDel==NULL) return m_head;


	if(nodeToDel==m_tail)
	{
		m_tail=node;
		m_tail->m_next=NULL;
	}
	else
	{
		node->m_next=nodeToDel->m_next;
	}

	delete nodeToDel->m_sprite;
	delete nodeToDel;

	return node;
}

uchar CLinkedSprites::CountSprite()
{
	uchar count=0;
	for(CNode* curNode=m_head; curNode!=NULL; curNode=curNode->m_next)
	{
		count++;
	}

	return count;
}

uchar CLinkedSprites::GetIndice(CBaseSpr* ptSpr)
{
	uchar count=0;
	for(CNode* curNode=m_head; curNode!=NULL && curNode->m_sprite!=ptSpr; curNode=curNode->m_next)
	{
		count++;
	}

	return count;

}

CBaseSpr* CLinkedSprites::GetSprite(uchar indice)
{
	uchar count=0;
	CNode* curNode;

	for(curNode=m_head; curNode!=NULL && count!=indice; curNode=curNode->m_next)
	{
		count++;
	}
	
	return curNode->m_sprite;
}