/****************************************************
 *	Original name : CW_PAttr.cpp					*
 *													*
 *	Class for using Pulsed Attractor.				*
 *													*
 *													*
 *	June 2003, LeadFiner							*
 ****************************************************/
#include "CW_PAttr.h"
#include "Data.h"
#include "Map.h"
#include "CLinkedS.h"
#include "DrwSprSW.h"
#include "Keys.h"
#include "CHero.h"
#include "CItem.h"
#include "CEnemy.h"

/********************************************************************************
 *																				*
 *						Class constructors										*
 *																				*
 ********************************************************************************/

CW_PAttr::CW_PAttr()
	:CWeapon(OBJECT_PATTR
	,8,8,8,8	//no collision
	,PATTR_SPR_ID_BOTTOM
	,PATTR_SPR_ID_LEFT
	,PATTR_SPR_ID_TOP
	,PATTR_SPR_ID_RIGHT
	,PATTR_DAMAGE)
{
}

/********************************************************************************
 *																				*
 *						Class methodes											*
 *																				*
 ********************************************************************************/

void CW_PAttr::StateActionRun()
{
	m_direction=hero.m_direction;

	SetMarginRect(PATTR_MARGE_BOTTOM,PATTR_MARGE_LEFT,PATTR_MARGE_TOP,PATTR_MARGE_RIGHT);

	switch(m_direction)
	{
	case DIR_BOTTOM:
		SetPos(hero.m_posX,hero.m_posY+6);
		break;
	case DIR_LEFT:
		SetPos(hero.m_posX-6,hero.m_posY);
		break;
	case DIR_TOP:
		SetPos(hero.m_posX,hero.m_posY-6);
		break;
	case DIR_RIGHT:
		SetPos(hero.m_posX+6,hero.m_posY);
		break;
	}

	uchar n=0;
	uchar tmpProp;
	
	do{
		m_destPosX=m_posX-mapPosX;
		m_destPosY=m_posY-mapPosY;
		
		switch(m_direction)
		{
		case DIR_BOTTOM:
			m_marginRect.bottom--;
			m_destPosY+=8-m_marginRect.bottom-2;
			break;
		case DIR_LEFT:
			m_marginRect.left--;
			m_destPosX+=m_marginRect.left-4;
			break;
		case DIR_TOP:
			m_marginRect.top--;
			m_destPosY+=m_marginRect.top-4;
			break;
		case DIR_RIGHT:
			m_marginRect.right--;
			m_destPosX+=8-m_marginRect.right-2;
			break;
		}
		SetMarginRect(m_marginRect.bottom,m_marginRect.left,m_marginRect.top,m_marginRect.right);
		n++;
		tmpProp=PropIs();
	}while(n<PATTR_DISTANCE && tmpProp!=P_STOP && (tmpProp<P_JUMP || tmpProp>=P_JUMP+MAP_JUMP_MAX) && IsInMap());

	if(!Touche(KEY_WEAPON_1)) 
	{
		m_stateAction=ACTION_END;
		TestAction();	//to test square property
	}

	//test caught items
	CNode* curNode=items.m_head;
	for(; curNode!=NULL; curNode=curNode->m_next)
	{
		if(!curNode->m_sprite->IsEnable()) continue;
		
		//test collision with attractor
		if(IsItemForKliz(curNode->m_sprite->m_objectID) && TestCollision((CSprite*)curNode->m_sprite))
		{
			((CItem*)curNode->m_sprite)->MoveTo((m_direction+2)%4);
		}
	}

	//test repulsed enemies
	curNode=enemies.m_head;
	for(; curNode!=NULL; curNode=curNode->m_next)
	{
		if(!curNode->m_sprite->IsEnable()) continue;
		
		//test collision with attractor
		if(TestCollision((CSprite*)curNode->m_sprite))
		{
			((CEnemy*)curNode->m_sprite)->MoveTo(m_direction);
		}
	}
	
	

	if(n<PATTR_DISTANCE && IsInMap() && (tmpProp<P_JUMP || tmpProp>=P_JUMP+MAP_JUMP_MAX)) 
	{
		//move Kliz if the Pulsed Attractor has caught a P_STOP square
		//keep current position
		short keepPosX=hero.m_posX;
		short keepPosY=hero.m_posY;
		
		switch(m_direction)
		{
		case DIR_BOTTOM:
			hero.m_posY++;
			break;
		case DIR_LEFT:
			hero.m_posX--;
			break;
		case DIR_TOP:
			hero.m_posY--;
			break;
		case DIR_RIGHT:
			hero.m_posX++;
			break;
		}
		
		if(hero.PropIs()==P_STOP) hero.SetPos(keepPosX, keepPosY);
	}

	//disable collision
	SetMarginRect(8,8,8,8);
}

void CW_PAttr::Draw()
{
	CWeapon::Draw();
	
	uchar sprDestID[4];
	sprDestID[DIR_BOTTOM]	=PATTR_D_SPR_ID_BOTTOM;
	sprDestID[DIR_LEFT]		=PATTR_D_SPR_ID_LEFT;
	sprDestID[DIR_TOP]		=PATTR_D_SPR_ID_TOP;
	sprDestID[DIR_RIGHT]	=PATTR_D_SPR_ID_RIGHT;
	
	DrawM21Spr_C(&memSprItem, m_destPosX, m_destPosY, sprDestID[m_direction],SEG1);
}

