/**
* @package Draw
*/
/****************************************************
 *	Part of Shadow World project					*
 *													*
 *	Original name : Draw.h							*
 *													*
 *	Drawing functions.								*
 *													*
 *													*
 *	May 2003, LeadFiner								*
 *	LeadFiner@wanadoo.fr							*
 ****************************************************/
#pragma inline

#if !defined DRAW
#define DRAW

#include "Types2.h"
#include "CSprite.h"

/********************************************************************************
 *																				*
 *						Data declarations and Defines							*
 *																				*
 ********************************************************************************/

//offset of window sprites
#define OFFSET_WND				96				//offset in tab SpriteFont for window sprites
#define OFFSET_WND_TL			OFFSET_WND+0	//offset in tab SpriteFont for window top-left sprites
#define OFFSET_WND_TR			OFFSET_WND+1	//offset in tab SpriteFont for window top-right sprites
#define OFFSET_WND_BR			OFFSET_WND+2	//offset in tab SpriteFont for window bottom-right sprites
#define OFFSET_WND_BL			OFFSET_WND+3	//offset in tab SpriteFont for window bottom-left sprites

#define OFFSET_WND_T			OFFSET_WND+4	//offset in tab SpriteFont for window top sprites
#define OFFSET_WND_R			OFFSET_WND+5	//offset in tab SpriteFont for window right sprites
#define OFFSET_WND_B			OFFSET_WND+6	//offset in tab SpriteFont for window bottom sprites
#define OFFSET_WND_L			OFFSET_WND+7	//offset in tab SpriteFont for window left sprites

#define OFFSET_WND_WHT			OFFSET_WND+8	//offset in tab SpriteFont for window white sprites

#define OFFSET_WND_ARR_B		OFFSET_WND+9	//offset in tab SpriteFont for window bottom arrow

//offset X and Y for printing sprites
#define DECAY_X_FONT_SMALL		5
#define DECAY_X_FONT_CAPITAL	8
#define DECAY_X_FONT_FIGURE		4
#define DECAY_X_FONT_SPACE		3
#define DECAY_Y_FONT			8

//for DisplayTextWindow(...)
#define SPEED_DISP_TXT			16		//number of steps for opening window
#define SPEED_ARROW_FLASH		512		//speed of the arrow's flash
#define STATE_PRINT				0		//for very small automat :-) in DisplayTextWindow(...)
#define STATE_WAIT				1		//for very small automat :-) in DisplayTextWindow(...)

#define DISP_WND_BOTTOM			64		//to display text on the bottom of screen
#define	DISP_WND_TOP			0		//to display text on the top of screen
#define	DISP_POS_LEFT			0		//position of the left of the window
#define	DISP_POS_RIGHT			128		//position of the right of the window

#define	DISP_MAX_LINE			4		//how many lines can be displayed in the same window

#define CHOOSE_NOTHING			-1		//no choose answer
#define CHOOSE_QUIT				0		//quit without choose answer
#define CHOOSE_1				1		//quit with choice 1
#define CHOOSE_2				2		//quit with choice 2
#define CHOOSE_3				3		//quit with choice 3
#define CHOOSE_4				4		//quit with choice 4
#define CHOOSE_5				5		//quit with choice 5
#define CHOOSE_6				6		//quit with choice 6
#define CHOOSE_7				7		//quit with choice 7
#define CHOOSE_8				8		//quit with choice 8
#define CHOOSE_9				9		//quit with choice 9
#define CHOOSE_0				10		//quit with choice 0


//for status bar
#define OFFSET_BAR				52
#define OFFSET_BAR_TL			OFFSET_BAR+0	//offset in tab SpriteMap for window top-left sprites
#define OFFSET_BAR_TR			OFFSET_BAR+1	//offset in tab SpriteMap for window top-right sprites
#define OFFSET_BAR_BR			OFFSET_BAR+17	//offset in tab SpriteMap for window bottom-right sprites
#define OFFSET_BAR_BL			OFFSET_BAR+16	//offset in tab SpriteMap for window bottom-left sprites
#define OFFSET_BAR_R			OFFSET_BAR+3	//offset in tab SpriteMap for window right sprites
#define OFFSET_BAR_L			OFFSET_BAR+18	//offset in tab SpriteMap for window left sprites

#define OFFSET_BAR_HEART		240				//offset in tab SpriteItem for heart sprites

/********************************************************************************
 *																				*
 *						Function Prototypes										*
 *																				*
 ********************************************************************************/

extern void DrawMap(ushort x, ushort y);
extern void DrawNextMapFX();
extern void DrawValueLayer(ushort value, ushort nLine, ushort segm=SEG1/*default*/);
extern void DrawScene();
extern void DrawStatusBar();
extern BOOL DrawChar(char* x, char* y, char c, ushort segm=SEG1/*default*/); //return TRUE if it is a new line (c='\n')
extern void DrawText(char x, char y, char* text, short segm=SEG1/*default*/);
extern void DrawWindow(char x1, char y1, uchar x2, uchar y2, short segm=SEG1/*default*/);
extern void DrawOpeningWindow(char x1, char y1, uchar x2, uchar y2, short segm=SEG1/*default*/);
extern char DisplayTextWindow(char posWnd, char* text, BOOL redrawScene=TRUE);

extern void CenterMapTo(CSprite tSpr);
#endif