/****************************************************
 *	Original name : LoadFile.cpp					*
 *													*
 *	Functions for finding, loading files.			*
 *													*
 *													*
 *	May 2003, LeadFiner								*
 ****************************************************/

#include "LoadFile.h"
#include "Graphic.h"
#include "Data.h"
#include "Draw.h"
#include "Keys.h"
#include "Anim.h"
#include "Map.h"
//#include <stdio.h>
//#include <conio.h>
#include <dos.h>
#include <share.h>
#include <fcntl.h>
#include <mem.h>
#include "memzones.h"
#include "CLinkedS.h"
#include "CEnemy.h"
#include "CItem.h"
#include "CWeapon.h"

/********************************************************************************
 *																				*
 *						Data Declarations										*
 *																				*
 ********************************************************************************/

int (*handler)(int,int,int,int)=error_handler;

/********************************************************************************
 *																				*
 *						Function Declarations									*
 *																				*
 ********************************************************************************/

int error_handler(int,int,int,int)
{
   hardresume(_HARDERR_FAIL);
   return 0;
}

BOOL FindDrive ()
{
	struct find_t ffblk;
	unsigned i,maxdrives=1;

	for(i=1 ; i<=maxdrives && _dos_findfirst(SW_EXE,FA_NORMAL,&ffblk)!=0; i++) _dos_setdrive(i,&maxdrives);

	if(open(SW_EXE, O_RDONLY | O_BINARY)<0) return FALSE;
	
	return TRUE;
}

void LoadMap(uchar mapID)
{
	int hfile;

	hfile=open(DEFAULT_WORLD, O_RDONLY | O_BINARY);

	//seach mapID en the index
	DWORD posMapInFile;

	lseek(hfile,mapID*sizeof(posMapInFile),SEEK_SET);
	read(hfile,&posMapInFile,sizeof(posMapInFile));

	//jump to the beginning of map with ID=mapID
	lseek(hfile, posMapInFile, SEEK_SET );

	//init curMap with 0
	memset(&curMap,0,sizeof(curMap));

	//read the map
	if(mapID==0) read(hfile, &startSquare,sizeof(startSquare));
	read(hfile, &curMap.dimX, sizeof(curMap.dimX));
	read(hfile, &curMap.dimY, sizeof(curMap.dimY));

	uchar buffer[MAP_SIZE_MAX];
	uchar compressTag;
	ushort sizeCompress;

	read(hfile, &compressTag, sizeof(compressTag));
	read(hfile, &sizeCompress, sizeof(sizeCompress));
	read(hfile, buffer,sizeCompress);
	DecompressBufferToBuffer(buffer, &curMap.tabSpr[0], sizeCompress, compressTag);

	read(hfile, &compressTag, sizeof(compressTag));
	read(hfile, &sizeCompress, sizeof(sizeCompress));
	read(hfile, buffer,sizeCompress);
	DecompressBufferToBuffer(buffer, &curMap.tabProp[0], sizeCompress, compressTag);

	
	read(hfile, &curMap.tabJump[0],sizeof(curMap.tabJump));
	read(hfile, &curMap.maxAni,sizeof(curMap.maxAni));
	read(hfile, &curMap.tabAni[0],(curMap.maxAni+1)*sizeof(curMap.tabAni[0]));
	read(hfile, &curMap.tabText[0],sizeof(curMap.tabText));

	close (hfile);

	//initialize constants
	posMapMaxPxX=curMap.dimX*8 - DIM_SCR_PX_X;
	posMapMaxPxY=curMap.dimY*8 - DIM_SCR_PX_Y;
	dimMapX=curMap.dimX;
	dimMapY=curMap.dimY;
	dimMapPxX=curMap.dimX*8;
	dimMapPxY=curMap.dimY*8;
	curMap.mapID=mapID;

	//update enemies, items and sceneries
	ScanMapEnemy();
	ScanMapItem();
	UpdateEnemy();
	UpdateItem();
	ScanMapScenery();
	if(hero.m_weapon1!=NULL)
	{
		hero.m_weapon1->SetDisable();
		hero.m_weapon1->CWeapon::StateActionForceEnd();
	}
}

void LoadBitmap(uchar *file)
{
	if(curBmpFile==file) return;	//don't reload same bitmap

	DecompressFileToMemzone(file, &memBitmap);

	curBmpFile=file;
}

void DecompressFileToMemzone(uchar *file, memory_zone *memz)
{
	int hfile;
	uchar compressTag;
	ushort sizeCompress;
	uchar buffer[SIZE_TAB_MAX];

	hfile=open(file, O_RDONLY | O_BINARY);

	read(hfile, &compressTag, sizeof(compressTag));
	read(hfile, &sizeCompress, sizeof(sizeCompress));

	read(hfile, buffer, sizeCompress);
	DecompressBufferToMemzone(buffer, memz, sizeCompress, compressTag);

	close (hfile);
}

void DecompressBufferToBuffer(uchar* srcBuffer, uchar* dstBuffer, ushort sizeCompress, uchar compressTag)
{
	uchar tmpByte;
	uchar tmpMatch=1;
	ushort tmpPos=0;
	ushort tmpPosMem=0;

	do
	{
		tmpByte=srcBuffer[tmpPos];
		tmpPos++;

		if (tmpByte!=compressTag) 
		{
			dstBuffer[tmpPosMem]=tmpByte;
			tmpPosMem++;
			tmpMatch=1;
		}
		else
		{
			tmpMatch=srcBuffer[tmpPos];
			tmpPos++;
			tmpByte=srcBuffer[tmpPos];
			tmpPos++;
			memset(dstBuffer+tmpPosMem,tmpByte,tmpMatch);
			tmpPosMem+=tmpMatch;
		}
	}while(tmpPos<sizeCompress);
}

void DecompressBufferToMemzone(uchar* srcBuffer, memory_zone *memz, ushort sizeCompress, uchar compressTag)
{
	uchar tmpByte;
	uchar tmpMatch=1;
	ushort tmpPos=0;
	ushort tmpPosMem=0;
	uchar buffer[256];
	
	do
	{
		tmpByte=srcBuffer[tmpPos];
		tmpPos++;

		if (tmpByte!=compressTag) 
		{
			tmpPosMem+=write_mem_zone(memz,&tmpByte,tmpPosMem,sizeof(tmpByte));
			tmpMatch=1;
		}
		else
		{
			tmpMatch=srcBuffer[tmpPos];
			tmpPos++;
			tmpByte=srcBuffer[tmpPos];
			tmpPos++;
			memset(buffer,tmpByte,tmpMatch);
			tmpPosMem+=write_mem_zone(memz,&buffer,tmpPosMem,tmpMatch);
		}
	} while(tmpPos<sizeCompress);
}

char* LoadText(ushort txtID)
{
	int hfile;

	hfile=open(DEFAULT_TEXT, O_RDONLY | O_BINARY);


	ushort tmpPos1;
	ushort tmpPos2;

	lseek(hfile,txtID*sizeof(tmpPos1),SEEK_SET);
	read(hfile,&tmpPos1,sizeof(tmpPos1));
	read(hfile,&tmpPos2,sizeof(tmpPos2));
	

	lseek(hfile,tmpPos1,SEEK_SET);
	memset(curText,0,sizeof(curText));
	read(hfile,curText,tmpPos2-tmpPos1);

	close (hfile);

	return curText;
}

BOOL SaveGame(uchar slot)
{	
	char* saveName;

	switch(slot)
	{
	case 1:
		saveName=DEFAULT_MEM_SAVE1;
		break;
	case 2:
		saveName=DEFAULT_MEM_SAVE2;
		break;
	case 3:
		saveName=DEFAULT_MEM_SAVE3;
		break;
	default:
		return FALSE;
	}

	CNode* curNode;
	short curOffset=0;
	uchar countEnemies=enemies.CountSprite();
	uchar countItems=items.CountSprite();
	uchar countHeroItems=hero.m_listItems.CountSprite();
	uchar countHeroWeapons=hero.m_listWeapons.CountSprite();
	uchar indexWeapon1=hero.m_listWeapons.GetIndice(hero.m_weapon1);
	//uchar indexWeapon2=hero.m_listWeapons.GetIndice(hero.m_weapon2);
	uchar tmpVersion=SW_VERSION;


	//calcul size of save for constant informations
	short curSize=
		sizeof(curSize)
		+sizeof(tmpVersion)
		+sizeof(tabMapEnemyIsScan)
		+sizeof(tabMapItemIsScan)
		+sizeof(curMap.mapID)
		+sizeof(hero.m_posX)
		+sizeof(hero.m_posY)
		+sizeof(hero.m_life)
		+sizeof(hero.m_maxLife)
		+sizeof(lastInventory)
		+sizeof(lastInvSel);

	//update size for enemies
	curSize+=countEnemies*(sizeof(((CEnemy*)enemies.m_head->m_sprite)->m_objectID)
		+sizeof(((CEnemy*)enemies.m_head->m_sprite)->m_posX)
		+sizeof(((CEnemy*)enemies.m_head->m_sprite)->m_posY)
		+sizeof(((CEnemy*)enemies.m_head->m_sprite)->m_myMapID)
		+sizeof(((CEnemy*)enemies.m_head->m_sprite)->m_life)
		+sizeof(((CEnemy*)enemies.m_head->m_sprite)->m_timerEnable))+sizeof(countEnemies);

	//update size for items
	curSize+=countItems*(sizeof(((CItem*)items.m_head->m_sprite)->m_objectID)
		+sizeof(((CItem*)items.m_head->m_sprite)->m_posX)
		+sizeof(((CItem*)items.m_head->m_sprite)->m_posY)
		+sizeof(((CItem*)items.m_head->m_sprite)->m_myMapID)
		+sizeof(((CItem*)items.m_head->m_sprite)->m_timerEnable))+sizeof(countItems);

	//update size for hero's weapons
	curSize+=countHeroWeapons*(sizeof(((CWeapon*)hero.m_listWeapons.m_head->m_sprite)->m_objectID)
		+sizeof(((CWeapon*)hero.m_listWeapons.m_head->m_sprite)->m_extra))
		+sizeof(indexWeapon1)+sizeof(countHeroWeapons);

	//update size for hero's items
	curSize+=countHeroItems*(sizeof(((CItem*)hero.m_listItems.m_head->m_sprite)->m_objectID)
		+sizeof(((CItem*)hero.m_listItems.m_head->m_sprite)->m_extra)
		+sizeof(((CItem*)hero.m_listItems.m_head->m_sprite)->m_timerEnable))	//use as variable
		+sizeof(countHeroItems);

	//try to create zone or use it if it exist
	if(!UseMemZone(DEFAULT_MEM_ZONE, saveName, curSize+SIZE_ZONE,&memSave)) return FALSE;


	//save begin here !!!
	
	//write size of save file and SW version to verify
	curOffset+=write_mem_zone(&memSave, &curSize, curOffset, sizeof(curSize));
	curOffset+=write_mem_zone(&memSave, &tmpVersion, curOffset, sizeof(tmpVersion));

	//write extra informations
	curOffset+=write_mem_zone(&memSave, &lastInventory, curOffset, sizeof(lastInventory));
	curOffset+=write_mem_zone(&memSave, &lastInvSel, curOffset, sizeof(lastInvSel));
	
	//save scan tabs
	curOffset+=write_mem_zone(&memSave, tabMapEnemyIsScan, curOffset, sizeof(tabMapEnemyIsScan));
	curOffset+=write_mem_zone(&memSave, tabMapItemIsScan, curOffset, sizeof(tabMapItemIsScan));
	
	//save mapID
	curOffset+=write_mem_zone(&memSave, &curMap.mapID, curOffset, sizeof(curMap.mapID));

	//save hero informations
	curOffset+=write_mem_zone(&memSave, &(hero.m_posX), curOffset, sizeof(hero.m_posX));
	curOffset+=write_mem_zone(&memSave, &(hero.m_posY), curOffset, sizeof(hero.m_posY));
	curOffset+=write_mem_zone(&memSave, &(hero.m_life), curOffset, sizeof(hero.m_life));
	curOffset+=write_mem_zone(&memSave, &(hero.m_maxLife), curOffset, sizeof(hero.m_maxLife));

	//save enemies
	curOffset+=write_mem_zone(&memSave,&countEnemies, curOffset, sizeof(countEnemies));
	for(curNode=enemies.m_head; curNode!=NULL; curNode=curNode->m_next)
	{
		curOffset+=write_mem_zone(&memSave, &((CEnemy*)curNode->m_sprite)->m_objectID, curOffset, sizeof(((CEnemy*)curNode->m_sprite)->m_objectID));
		curOffset+=write_mem_zone(&memSave, &((CEnemy*)curNode->m_sprite)->m_posX, curOffset, sizeof(((CEnemy*)curNode->m_sprite)->m_posX));
		curOffset+=write_mem_zone(&memSave, &((CEnemy*)curNode->m_sprite)->m_posY, curOffset, sizeof(((CEnemy*)curNode->m_sprite)->m_posY));
		curOffset+=write_mem_zone(&memSave, &((CEnemy*)curNode->m_sprite)->m_myMapID, curOffset, sizeof(((CEnemy*)curNode->m_sprite)->m_myMapID));
		curOffset+=write_mem_zone(&memSave, &((CEnemy*)curNode->m_sprite)->m_life, curOffset, sizeof(((CEnemy*)curNode->m_sprite)->m_life));
		curOffset+=write_mem_zone(&memSave, &((CEnemy*)curNode->m_sprite)->m_timerEnable, curOffset, sizeof(((CEnemy*)curNode->m_sprite)->m_timerEnable));
	}

	//save items
	curOffset+=write_mem_zone(&memSave,&countItems, curOffset, sizeof(countItems));
	for(curNode=items.m_head; curNode!=NULL; curNode=curNode->m_next)
	{
		curOffset+=write_mem_zone(&memSave, &((CItem*)curNode->m_sprite)->m_objectID, curOffset, sizeof(((CItem*)curNode->m_sprite)->m_objectID));
		curOffset+=write_mem_zone(&memSave, &((CItem*)curNode->m_sprite)->m_posX, curOffset, sizeof(((CItem*)curNode->m_sprite)->m_posX));
		curOffset+=write_mem_zone(&memSave, &((CItem*)curNode->m_sprite)->m_posY, curOffset, sizeof(((CItem*)curNode->m_sprite)->m_posY));
		curOffset+=write_mem_zone(&memSave, &((CItem*)curNode->m_sprite)->m_myMapID, curOffset, sizeof(((CItem*)curNode->m_sprite)->m_myMapID));
		curOffset+=write_mem_zone(&memSave, &((CItem*)curNode->m_sprite)->m_timerEnable, curOffset, sizeof(((CItem*)curNode->m_sprite)->m_timerEnable));
	}

	//save hero weapons
	curOffset+=write_mem_zone(&memSave,&countHeroWeapons, curOffset, sizeof(countHeroWeapons));
	if(countHeroWeapons!=0)
	{
		curOffset+=write_mem_zone(&memSave,&indexWeapon1, curOffset, sizeof(indexWeapon1));
		//curOffset+=write_mem_zone(&memSave,&indexWeapon2, curOffset, sizeof(indexWeapon2));
		for(curNode=hero.m_listWeapons.m_head; curNode!=NULL; curNode=curNode->m_next)
		{
			curOffset+=write_mem_zone(&memSave, &((CWeapon*)curNode->m_sprite)->m_objectID, curOffset, sizeof(((CWeapon*)curNode->m_sprite)->m_objectID));
			curOffset+=write_mem_zone(&memSave, &((CWeapon*)curNode->m_sprite)->m_extra, curOffset, sizeof(((CWeapon*)curNode->m_sprite)->m_extra));
		}
	}

	//save hero items
	curOffset+=write_mem_zone(&memSave,&countHeroItems, curOffset, sizeof(countHeroItems));
	if(countHeroItems!=0)
	{
		for(curNode=hero.m_listItems.m_head; curNode!=NULL; curNode=curNode->m_next)
		{
			curOffset+=write_mem_zone(&memSave, &((CItem*)curNode->m_sprite)->m_objectID, curOffset, sizeof(((CItem*)curNode->m_sprite)->m_objectID));
			curOffset+=write_mem_zone(&memSave, &((CItem*)curNode->m_sprite)->m_extra, curOffset, sizeof(((CItem*)curNode->m_sprite)->m_extra));
			curOffset+=write_mem_zone(&memSave, &((CItem*)curNode->m_sprite)->m_timerEnable, curOffset, sizeof(((CItem*)curNode->m_sprite)->m_timerEnable));
		}
	}

	UpdateMemZones();
	UpdateItem();

	return TRUE;
}

BOOL LoadGame(uchar slot)
{
	char* saveName;

	switch(slot)
	{
	case 1:
		saveName=DEFAULT_MEM_SAVE1;
		break;
	case 2:
		saveName=DEFAULT_MEM_SAVE2;
		break;
	case 3:
		saveName=DEFAULT_MEM_SAVE3;
		break;
	default:
		return FALSE;
	}

	short curSize;


	curSize=search_mem_zone(DEFAULT_MEM_ZONE,saveName,&memSave);

	if (curSize<SIZE_SAVE+SIZE_ZONE) return FALSE;

	short curOffset=0;
	uchar i;
	uchar countEnemies;
	uchar countItems;
	uchar countHeroItems;
	uchar countHeroWeapons;
	uchar indexWeapon1;
	//uchar indexWeapon2;
	CBaseSpr* curSpr;
	uchar tmpObjID;
	uchar tmpVersion;
	short tmpPosX;
	short tmpPosY;
	
	//load size of save file and SW version to verify
	curOffset+=read_mem_zone(&memSave, &curSize, curOffset, sizeof(curSize));
	curOffset+=read_mem_zone(&memSave, &tmpVersion, curOffset, sizeof(tmpVersion));

	//verify size of save file
	if(curSize!=memSave.b_real_size || tmpVersion!=SW_VERSION) return FALSE;


	//load begin here !!!

	//read extra informations
	curOffset+=read_mem_zone(&memSave, &lastInventory, curOffset, sizeof(lastInventory));
	curOffset+=read_mem_zone(&memSave, &lastInvSel, curOffset, sizeof(lastInvSel));


	//load scan tabs
	curOffset+=read_mem_zone(&memSave, tabMapEnemyIsScan, curOffset, sizeof(tabMapEnemyIsScan));
	curOffset+=read_mem_zone(&memSave, tabMapItemIsScan, curOffset, sizeof(tabMapItemIsScan));

	//load mapID
	uchar tmpMapID;
	curOffset+=read_mem_zone(&memSave, &tmpMapID, curOffset, sizeof(tmpMapID));

	//load hero informations
	curOffset+=read_mem_zone(&memSave, &hero.m_posX, curOffset, sizeof(hero.m_posX));
	curOffset+=read_mem_zone(&memSave, &hero.m_posY, curOffset, sizeof(hero.m_posY));
	curOffset+=read_mem_zone(&memSave, &hero.m_life, curOffset, sizeof(hero.m_life));
	curOffset+=read_mem_zone(&memSave, &hero.m_maxLife, curOffset, sizeof(hero.m_maxLife));

	//load enemies
	curOffset+=read_mem_zone(&memSave,&countEnemies, curOffset, sizeof(countEnemies));
	for(i=0; i<countEnemies; i++)
	{
		curOffset+=read_mem_zone(&memSave, &tmpObjID, curOffset, sizeof(tmpObjID));
		curOffset+=read_mem_zone(&memSave, &tmpPosX, curOffset, sizeof(tmpPosX));
		curOffset+=read_mem_zone(&memSave, &tmpPosY, curOffset, sizeof(tmpPosY));

		curSpr=AddEnemy(tmpObjID,tmpPosX,tmpPosY);

		if(curSpr==NULL) return FALSE;

		curOffset+=read_mem_zone(&memSave, &((CEnemy*)curSpr)->m_myMapID, curOffset, sizeof(((CEnemy*)curSpr)->m_myMapID));
		curOffset+=read_mem_zone(&memSave, &((CEnemy*)curSpr)->m_life, curOffset, sizeof(((CEnemy*)curSpr)->m_life));
		curOffset+=read_mem_zone(&memSave, &((CEnemy*)curSpr)->m_timerEnable, curOffset, sizeof(((CEnemy*)curSpr)->m_timerEnable));
	}

	//load items
	curOffset+=read_mem_zone(&memSave,&countItems, curOffset, sizeof(countItems));
	for(i=0; i<countItems; i++)
	{
		curOffset+=read_mem_zone(&memSave, &tmpObjID, curOffset, sizeof(tmpObjID));
		curOffset+=read_mem_zone(&memSave, &tmpPosX, curOffset, sizeof(tmpPosX));
		curOffset+=read_mem_zone(&memSave, &tmpPosY, curOffset, sizeof(tmpPosY));
		
		curSpr=AddItem(tmpObjID,tmpPosX,tmpPosY);

		if(curSpr==NULL) return FALSE;

		curOffset+=read_mem_zone(&memSave, &((CItem*)curSpr)->m_myMapID, curOffset, sizeof(((CItem*)curSpr)->m_myMapID));
		curOffset+=read_mem_zone(&memSave, &((CItem*)curSpr)->m_timerEnable, curOffset, sizeof(((CItem*)curSpr)->m_timerEnable));
	}

	//load hero weapons
	curOffset+=read_mem_zone(&memSave,&countHeroWeapons, curOffset, sizeof(countHeroWeapons));
	if(countHeroWeapons!=0)
	{
		curOffset+=read_mem_zone(&memSave,&indexWeapon1, curOffset, sizeof(indexWeapon1));
		//curOffset+=read_mem_zone(&memSave,&indexWeapon2, curOffset, sizeof(indexWeapon2));
		for(i=0; i<countHeroWeapons; i++)
		{
			curOffset+=read_mem_zone(&memSave, &tmpObjID, curOffset, sizeof(tmpObjID));
			
			curSpr=AddHeroWeapon(tmpObjID);

			if(curSpr==NULL) return FALSE;

			curOffset+=read_mem_zone(&memSave, &((CWeapon*)curSpr)->m_extra, curOffset, sizeof(((CWeapon*)curSpr)->m_extra));

			if(i==indexWeapon1) hero.m_weapon1=(CWeapon*)curSpr;
		}
	}

	//load hero items
	curOffset+=read_mem_zone(&memSave,&countHeroItems, curOffset, sizeof(countHeroItems));
	if(countHeroItems!=0)
	{
		for(i=0; i<countHeroItems; i++)
		{
			curOffset+=read_mem_zone(&memSave, &tmpObjID, curOffset, sizeof(tmpObjID));
			
			curSpr=AddHeroItem(tmpObjID);

			if(curSpr==NULL) return FALSE;

			curOffset+=read_mem_zone(&memSave, &((CItem*)curSpr)->m_extra, curOffset, sizeof(((CItem*)curSpr)->m_extra));
			curOffset+=read_mem_zone(&memSave, &((CItem*)curSpr)->m_timerEnable, curOffset, sizeof(((CItem*)curSpr)->m_timerEnable));
		}
	}

	
	LoadMap(tmpMapID);
	hero.SetGhost();

	UpdateMemZones();

	return TRUE;
}

BOOL SaveConfig()
{
	//alloc for config
	if(!UseMemZone(DEFAULT_MEM_ZONE, DEFAULT_MEM_CFG, SIZE_CFG+SIZE_ZONE,&memCFG)) return FALSE;


	write_mem_zone(&memCFG, &contrastDB, 0, sizeof(contrastDB));

	UpdateMemZones();

	return TRUE;
}

BOOL LoadConfig()
{
	long curSize;

	curSize=search_mem_zone(DEFAULT_MEM_ZONE,DEFAULT_MEM_CFG,&memCFG);

	if (curSize<SIZE_CFG+SIZE_ZONE) return FALSE;

	read_mem_zone(&memCFG, &contrastDB, 0, sizeof(contrastDB));

	UpdateMemZones();

	return TRUE;
}
