/****************************************************
 *	Original name : Map.cpp							*
 *													*
 *	Functions for map.								*
 *													*
 *													*
 *	May 2003, LeadFiner								*
 ****************************************************/

#include "Map.h"
#include "Data.h"
#include "Keys.h"
#include "System2.h"
#include "CPerso.h"
#include "Draw.h"

#include "CEnemy.h"
#include "CE_Larva.h"
#include "CE_Croc.h"
#include "CE_Boun.h"
#include "CE_Folow.h"
#include "CE_Shad1.h"
#include "CE_Bloc.h"
#include "CE_Wall.h"

#include "CI_Push.h"
#include "CI_Key.h"
#include "CI_Lock.h"
#include "CI_ShinS.h"
#include "CI_Heart.h"
#include "CI_Buyer.h"
#include "CI_Plant.h"
#include "CI_Crack.h"
#include "CI_MTele.h"
#include "CI_HrtUp.h"
#include "CI_Fline.h"

#include "CI_Sword.h"
#include "CI_Arrow.h"
#include "CI_Bomb.h"
#include "CI_Boomg.h"
#include "CI_PAttr.h"

#include "CW_Sword.h"
#include "CW_Arrow.h"
#include "CW_Bomb.h"
#include "CW_Boomg.h"
#include "CW_PAttr.h"

#include "LoadFile.h"
#include "CLinkedS.h"

#include "CScenery.h"
#include "CS_Storm.h"
#include "CS_Halo.h"

/********************************************************************************
 *																				*
 *						Function Declarations									*
 *																				*
 ********************************************************************************/

uchar TestProp(uchar curProp, uchar prevReturnProp)
{
	//normal test
	switch(curProp)
	{
	case P_TERRAN:
		return prevReturnProp;
	case P_STOP:
		return P_STOP;
	}

	//priority
	switch(prevReturnProp)
	{
	case P_STOP:case P_WATER:
		return prevReturnProp;
	}

	return curProp;
}

void ScanMapEnemy()
{
	if(GetMapEnemyIsScan(GetMapID())) return;

	DeleteSpritesInMap(&enemies, GetMapID());	//to avoid clones !!!

	//map is scan, we remind that
	SetMapEnemyIsScan(GetMapID(), TRUE);

	for(ushort indice=0; indice<dimMapX*dimMapY; indice++)
	{
		AddEnemy(GetProp(indice),CaseToPosPxX(indice), CaseToPosPxY(indice));
	}
}

void UpdateEnemy()
{
	CNode* curNode;

	//disable all sprites
	for(curNode=enemies.m_head; curNode!=NULL; curNode=curNode->m_next)
	{
		curNode->m_sprite->SetDisable();
	}

	//enable only sprites in map
	for(curNode=enemies.m_head; curNode!=NULL; curNode=curNode->m_next)
	{
		if(((CSprite*)curNode->m_sprite)->m_myMapID==GetMapID())
		{
			curNode->m_sprite->SetEnable();
		}
	}

	//search for old sprites
	BOOL haveDeleted;
	do{
		haveDeleted=FALSE;
		for(curNode=enemies.m_head; curNode!=NULL; curNode=curNode->m_next)
		{
			if(((CEnemy*)curNode->m_sprite)->CountDisable())
			{
				SetMapEnemyIsScan(((CSprite*)curNode->m_sprite)->m_myMapID, FALSE); //we must scan map next time
				DeleteSpritesInMap(&enemies, ((CSprite*)curNode->m_sprite)->m_myMapID);
				haveDeleted=TRUE;
				break;
			}
		}
	} while(haveDeleted);
}

void ScanMapScenery()
{
	ushort indice;
	short tmpPosX;
	short tmpPosY;

	//delete all old scenery
	scenery.DeleteAll();

	//create new scenery
	for(indice=0; indice<dimMapX*dimMapY; indice++)
	{


		tmpPosX=CaseToPosPxX(indice);
		tmpPosY=CaseToPosPxY(indice);

		switch(GetProp(indice))
		{
		case SCENERY_TREE:
			scenery.AddSprite(new CScenery(tmpPosX,tmpPosY, SPR_ITEM_TREE_L,ANI_LEN_TREE, ANI_SPEED_TREE_L));
			scenery.AddSprite(new CScenery(tmpPosX+8,tmpPosY, SPR_ITEM_TREE_R,ANI_LEN_TREE, ANI_SPEED_TREE_R));
			break;
		case SCENERY_TREE2:
			scenery.AddSprite(new CScenery(tmpPosX,tmpPosY, SPR_ITEM_TREE2_L));
			scenery.AddSprite(new CScenery(tmpPosX+8,tmpPosY, SPR_ITEM_TREE2_R));
			break;
		case SCENERY_ROCK:
			scenery.AddSprite(new CScenery(tmpPosX,tmpPosY, SPR_ITEM_ROCK_L));
			scenery.AddSprite(new CScenery(tmpPosX+8,tmpPosY, SPR_ITEM_ROCK_R));
			break;
		case SCENERY_COLUMN:
			scenery.AddSprite(new CScenery(tmpPosX,tmpPosY, SPR_ITEM_COLUMN));
			break;
		}
	}

	//create special scenery (at top of list)
	for(indice=0; indice<dimMapX*dimMapY; indice++)
	{
		tmpPosX=CaseToPosPxX(indice);
		tmpPosY=CaseToPosPxY(indice);

		switch(GetProp(indice))
		{
		case SCENERY_STORM:
			scenery.AddSprite(new CS_Storm());
			break;
		case SCENERY_HALO:
			scenery.AddSprite(new CS_Halo());
			break;
		}
	}
}

void ScanMapItem()
{
	ushort indice;

	//object are scanned one time
	if(!GetMapItemIsScan(GetMapID()))
	{
		//map is scan, we remind that
		SetMapItemIsScan(GetMapID(), TRUE);


		for(indice=0; indice<dimMapX*dimMapY; indice++)
		{
			switch(GetProp(indice))
			{
			case OBJECT_PUSH:case OBJECT_BUYER:case OBJECT_PLANT:
				break;
			default:
				AddItem(GetProp(indice), CaseToPosPxX(indice), CaseToPosPxY(indice));
			}
		}
	}

	DeleteSpecialItemInMap(GetMapID()); //to avoid clones when load...

	//we load particular items, must be initialised
	for(indice=0; indice<dimMapX*dimMapY; indice++)
	{
		switch(GetProp(indice))
		{
		case OBJECT_PUSH:case OBJECT_BUYER:case OBJECT_PLANT:
			AddItem(GetProp(indice), CaseToPosPxX(indice),CaseToPosPxY(indice));
			break;
		}
	}
}

void UpdateItem()
{
	CNode* curNode=items.m_head;

	//disable all sprites
	for(curNode=items.m_head; curNode!=NULL; curNode=curNode->m_next)
	{
		curNode->m_sprite->SetDisable();
	}

	//enable only sprites in map
	for(curNode=items.m_head; curNode!=NULL; curNode=curNode->m_next)
	{
		if(((CSprite*)curNode->m_sprite)->m_myMapID==GetMapID())
		{
			curNode->m_sprite->SetEnable();
		}
	}

	//search for special objects and delete it in other maps
	for(short tmpMapID=0; tmpMapID<256; tmpMapID++)
	{
		if(tmpMapID!=GetMapID()) DeleteSpecialItemInMap(tmpMapID);
	}

	//search for old sprites
	curNode=items.m_head;
	while(curNode!=NULL)
	{
		if(((CItem*)curNode->m_sprite)->CountDisable())
		{		
			curNode=items.DeleteSprite(curNode);
		}
		else curNode=curNode->m_next;
	}
}

void DeleteSpecialItemInMap(uchar mapID)
{
	CNode* curNode=items.m_head;
	
	while(curNode!=NULL)
	{
		if(((CSprite*)curNode->m_sprite)->m_myMapID==mapID)
		{
			switch(curNode->m_sprite->m_objectID)
			{
			case OBJECT_PUSH:case OBJECT_BUYER:case OBJECT_PLANT:
				curNode=items.DeleteSprite(curNode);
				break;
			default:
				curNode=curNode->m_next;
			}
		}
		else curNode=curNode->m_next;
	}
}

void DeleteSpritesInMap(CLinkedSprites* sprList, uchar mapID)
{
	CNode* curNode=sprList->m_head;

	while(curNode!=NULL)
	{
		if(((CSprite*)curNode->m_sprite)->m_myMapID==mapID)
		{
			curNode=sprList->DeleteSprite(curNode);
		}
		else curNode=curNode->m_next;
	}
}

CEnemy* AddEnemy(uchar enemyID, short posX, short posY)
{
	switch(enemyID)
	{
	case ENEMY_LARVA:
		return (CEnemy*) (enemies.AddSprite(new CE_Larva(posX,posY)));
	case ENEMY_CROC:
		return (CEnemy*) (enemies.AddSprite(new CE_Croc(posX,posY)));
	case ENEMY_BOUNC:
		return (CEnemy*) (enemies.AddSprite(new CE_Boun(posX,posY)));
	case ENEMY_FOLOW:
		return (CEnemy*) (enemies.AddSprite(new CE_Folow(posX,posY)));
	case ENEMY_SHADOW1:
		return (CEnemy*) (enemies.AddSprite(new CE_Shad1(posX,posY)));
	case ENEMY_WALL:
		return (CEnemy*) (enemies.AddSprite(new CE_Wall(posX,posY)));
	}

	return NULL;
}

CItem* AddItem(uchar itemID, short posX, short posY)
{
	if(itemID>=OBJECT_KEY && itemID<(OBJECT_KEY+MAP_KEY_MAX))
	{
		return (CItem*) (items.AddSprite(new CI_Key(itemID, posX, posY)));
	}
	
	if(itemID>=OBJECT_LOCK && itemID<(OBJECT_LOCK+MAP_KEY_MAX))
	{
		return (CItem*) (items.AddSprite(new CI_Lock(itemID, posX, posY)));
	}

	switch(itemID)
	{
	case OBJECT_PUSH:
		return (CItem*) (items.AddSprite(new CI_Push(posX, posY)));
	case OBJECT_SHINYS:
		return (CItem*) (items.AddSprite(new CI_ShinyS(posX, posY)));
	case OBJECT_HEART:
		return (CItem*) (items.AddSprite(new CI_Heart(posX, posY)));
	case OBJECT_BUYER:
		return (CItem*) (items.AddSprite(new CI_Buyer(posX, posY)));
	case OBJECT_PLANT:
		return (CItem*) (items.AddSprite(new CI_Plant(posX, posY)));
	case OBJECT_CRACK:
		return (CItem*) (items.AddSprite(new CI_Crack(posX, posY)));
	case OBJECT_MTELE:
		return (CItem*) (items.AddSprite(new CI_MTele(posX, posY)));
	case OBJECT_HEART_UP:
		return (CItem*) (items.AddSprite(new CI_HrtUp(posX, posY)));
	case OBJECT_FLINE:
		return (CItem*) (items.AddSprite(new CI_Fline(posX, posY)));

	case ENEMY_BLOC:	//special enemy !!!
		return (CItem*) (enemies.AddSprite(new CE_Bloc(posX,posY)));

	case OBJECT_SWORD:
		return (CItem*) (items.AddSprite(new CI_Sword(posX, posY)));
	case OBJECT_ARROW:
		return (CItem*) (items.AddSprite(new CI_Arrow(posX, posY)));
	case OBJECT_BOMB:
		return (CItem*) (items.AddSprite(new CI_Bomb(posX, posY)));
	case OBJECT_BOOMG:
		return (CItem*) (items.AddSprite(new CI_Boomg(posX, posY)));
	case OBJECT_PATTR:
		return (CItem*) (items.AddSprite(new CI_PAttr(posX, posY)));

	}

	return NULL;
}

void GenerateRndItem(short posX, short posY)
{
	CItem* newItem=NULL;

	switch(rand()%64)
	{
	case 0:case 1:case 2:case 3:case 4:case 5:case 6:case 7:case 8:case 9:case 10:case 11:case 12:case 13:case 14:
		newItem=AddItem(OBJECT_SHINYS, posX, posY);
		break;
	case 15:case 16:
		newItem=AddItem(OBJECT_HEART, posX, posY);
		break;
	case 18:
		newItem=AddItem(OBJECT_ARROW, posX, posY);
		break;
	case 20:
		newItem=AddItem(OBJECT_BOMB, posX, posY);
		break;
	}

	if(newItem!=NULL) newItem->m_timerEnable=0;
}

CWeapon* AddHeroWeapon(uchar weaponID, uchar extra)
{
	CNode* curNode;

	//search weapon to increase m_extra if it already exist
	for(curNode=hero.m_listWeapons.m_head; curNode!=NULL && curNode->m_sprite->m_objectID!=weaponID; curNode=curNode->m_next);

	if(curNode!=NULL)
	{
		if((((CWeapon*)curNode->m_sprite)->m_extra+extra)>255) ((CWeapon*)curNode->m_sprite)->m_extra=255;
		else ((CWeapon*)curNode->m_sprite)->m_extra+=extra;

		return (CWeapon*)(curNode->m_sprite);
	}

	//else create new weapon
	CWeapon* newWeapon=NULL;
	switch(weaponID)
	{
	case OBJECT_SWORD:
		newWeapon=(CWeapon*) (hero.m_listWeapons.AddSprite(new CW_Sword()));
		break;
	case OBJECT_ARROW:
		newWeapon=(CWeapon*) (hero.m_listWeapons.AddSprite(new CW_Arrow()));
		break;
	case OBJECT_BOMB:
		newWeapon=(CWeapon*) (hero.m_listWeapons.AddSprite(new CW_Bomb()));
		break;
	case OBJECT_BOOMG:
		newWeapon=(CWeapon*) (hero.m_listWeapons.AddSprite(new CW_Boomg()));
		break;
	case OBJECT_PATTR:
		newWeapon=(CWeapon*) (hero.m_listWeapons.AddSprite(new CW_PAttr()));
		break;
	}

	if(newWeapon!=NULL) newWeapon->m_extra=extra;

	return newWeapon;
}

CItem* AddHeroItem(uchar itemID, uchar extra)
{
	CNode* curNode;

	//search item to increase m_extra if it already exist
	for(curNode=hero.m_listItems.m_head; curNode!=NULL && curNode->m_sprite->m_objectID!=itemID; curNode=curNode->m_next);

	if(curNode!=NULL)
	{
		if((((CItem*)curNode->m_sprite)->m_extra+extra)>255)
			((CItem*)curNode->m_sprite)->m_extra=255;
		else
			((CItem*)curNode->m_sprite)->m_extra+=extra;
		return (CItem*)(curNode->m_sprite);
	}

	//else create new item
	CItem* newItem=NULL;
	if(itemID>=OBJECT_KEY && itemID<(OBJECT_KEY+MAP_KEY_MAX))
	{
		newItem=(CItem*) (hero.m_listItems.AddSprite(new CI_Key(itemID, 0, 0)));
	}
	
	switch(itemID)
	{
	case OBJECT_SHINYS:
		newItem=(CItem*) (hero.m_listItems.AddSprite(new CI_ShinyS(0, 0)));
		break;
	case OBJECT_HEART:
		newItem=(CItem*) (hero.m_listItems.AddSprite(new CI_Heart(0, 0)));
		break;
	case OBJECT_MTELE:
		newItem=(CItem*) (hero.m_listItems.AddSprite(new CI_MTele(0, 0)));
		break;
	case OBJECT_FLINE:
		newItem=(CItem*) (hero.m_listItems.AddSprite(new CI_Fline(0, 0)));
		break;
	}

	if(newItem!=NULL) newItem->m_extra=extra;

	return newItem;
}

void KillAllInMap(uchar mapID, char enemyID/*default=-1*/)
{
	for(CNode* curNode=enemies.m_head; curNode!=NULL; curNode=curNode->m_next)
	{
		if(((CSprite*)curNode->m_sprite)->m_myMapID==mapID)
		{
			if(enemyID==-1 || curNode->m_sprite->m_objectID==enemyID) ((CEnemy*)curNode->m_sprite)->SetKilled();
		}
	}
}

uchar CountEnemyInMap(uchar mapID)
{
	uchar count=0;

	for(CNode* curNode=enemies.m_head; curNode!=NULL; curNode=curNode->m_next)
	{
		if(((CSprite*)curNode->m_sprite)->m_myMapID==mapID)
		{
			count++;
		}
	}

	return count;
}

void GlobalUpdate()
{
	CNode* curNode=enemies.m_head;

	//test enemies
	while(curNode!=NULL)
	{
		if(!curNode->m_sprite->IsEnable())
		{//if sprite is disable
			((CEnemy*)curNode->m_sprite)->CountDisable();
			curNode=curNode->m_next;
			continue;	
		}

		//enemies is died ???
		if(((CEnemy*)curNode->m_sprite)->m_lifeState==LIFE_STATE_DIED)
		{
			curNode=enemies.DeleteSprite(curNode);
			continue;
		}

		//if sprite is killed, pass next sprite
		if(((CEnemy*)curNode->m_sprite)->m_lifeState==LIFE_STATE_KILL) 
		{
			curNode=curNode->m_next;
			continue; //nothing lose if enemy is killed
		}

		//if sprite is enable and not killed
		((CEnemy*) curNode->m_sprite)->AutoMove();

		//test collision with weapon
		if(hero.m_weapon1!=NULL && hero.m_weapon1->IsEnable())
		{
			//test collision with Weapon1
			if(((CEnemy*)curNode->m_sprite)->m_lifeState==LIFE_STATE_ALIVE && hero.m_weapon1->TestCollision((CSprite*)curNode->m_sprite))
			{
				//damage enemy
				if(((CEnemy*)curNode->m_sprite)->IncLife(hero.m_weapon1->m_damage)<=0) 
				{
					((CEnemy*)curNode->m_sprite)->SetKilled();
				}
				else
				{
					((CEnemy*)curNode->m_sprite)->SetDamaged();
				}
				
				hero.m_weapon1->StateActionForceEnd();
			}
		}

		//test collision with Hero
		if(hero.TestCollision((CSprite*)curNode->m_sprite))
		{
			hero.IncLife(((CEnemy*)curNode->m_sprite)->m_damage);

			FlyTo(2*hero.m_posX-curNode->m_sprite->m_posX
				, 2*hero.m_posY-curNode->m_sprite->m_posY
				,SCAN_STOP | SCAN_WATER);
			hero.SetGhost();
		}

		curNode=curNode->m_next;
	}


	//test items
	curNode=items.m_head;
	while(curNode!=NULL)
	{

		if(curNode->m_sprite->IsEnable()) 
		{
			//test collision with Hero
			if(hero.TestCollision((CSprite*)curNode->m_sprite))
			{	
				if(((CItem*)curNode->m_sprite)->Action()) 
				{	
					curNode=items.DeleteSprite(curNode);
					continue;
				}
			}
		}
		else ((CItem*)curNode->m_sprite)->CountDisable();


		curNode=curNode->m_next;
	}
}

BOOL IsItemForKliz(uchar objID)
{
	if(objID==OBJECT_SHINYS 
		|| objID==OBJECT_HEART 
		|| objID==OBJECT_BOMB 
		|| objID==OBJECT_ARROW
		|| objID==OBJECT_HEART_UP
		|| objID==OBJECT_PATTR
		|| objID==OBJECT_MTELE
		|| objID==OBJECT_FLINE
		|| (objID>=OBJECT_KEY && objID<OBJECT_KEY+MAP_KEY_MAX)) return TRUE;
	return FALSE;
}