/****************************************************
*	Original name : SW.cpp							*
*													*
*	The main program for Shadow World.				*
*													*
*													*
*	May 2003, LeadFiner								*
****************************************************/

#include "SW.h"
#include "LoadFile.h"
#include "Data.h"
#include "DrwSprSW.h"
#include "Anim.h"
#include "Draw.h"
#include "System2.h"
#include "Keys.h"
#include "Graphic.h"
#include <mem.h>
#include "Map.h"

#include "Chero.h"
#include "CLinkedS.h"
#include "CEnemy.h"
#include "CWeapon.h"

#include <STDLIB.h>
#include <TIME.h>
#include <dos.h>
#include <alloc.h>
#include "memzones.h"

//#include <conio.h>

/********************************************************************************
*																				*
*						Function Declarations									*
*																				*
********************************************************************************/

void InitProg()
{
	//init contrast
	contrastC3=GetContrast();
	contrastDB=REF_CONTRAST;

	//clear screen, buffers, and enemies
	ClearScreen(SEG1);
	ClearScreen(SEG0);

	/*//alloc memory for maps
	curMap.tabSpr=new uchar[MAP_SIZE_MAX];
	curMap.tabProp=new uchar[MAP_SIZE_MAX];*/


	//Disable the key buffer
	SetDisableKeyBuf();

	if(!FindDrive()) QuitGame();

	if(!AllocMemoryForSpr()) 
	{
		printf(DEFAULT_MEM_ERROR);
		while(!Touche(MKS_ESC));
		QuitGame();
	}
	
	//init random function
	randomize();

	//init video by loading config
	if(!LoadConfig())
	{
		if(!SaveConfig()) 
		{
			printf(DEFAULT_MEM_ERROR);
			while(!Touche(MKS_ESC));
			QuitGame();
		}
	}

	//message for loading
	printf(DEFAULT_LOADING);

	//load sprites, fonts and animations
	DecompressFileToMemzone(DEFAULT_SPR_FONT, &memSprFont);
	DecompressFileToMemzone(DEFAULT_SPR_ITEM, &memSprItem);
	DecompressFileToMemzone(DEFAULT_SPR_MAP, &memSprMap);

	//load intro
	LoadBitmap(DEFAULT_INTRO);

	//Set video mode
	SetMode(MODE_DB);
}

void InitGame()
{
	//init mapIDLoaded
	memset(tabMapEnemyIsScan,0,sizeof(tabMapEnemyIsScan));
	memset(tabMapItemIsScan,0,sizeof(tabMapItemIsScan));
		
	//load first map
	LoadMap(0);

	//init Hero
	hero.SetPos(CaseToPosPxX(startSquare),CaseToPosPxY(startSquare));
	hero.m_life=hero.m_maxLife;
	hero.SetGhost();

	//init variables
	lastInvSel=0;
	lastInventory=INV_NONE;	//init inventory to help
}

void main(/*int argc, char* argv[]*/)
{
	//error exception
	harderr(handler);

	//init
	InitProg();
	InitGame();
	Intro();
	
	uchar rtnChoice=StartMenu();
	
	do
	{
		if(rtnChoice==CHOICE_LOAD) 
		{
			ContrastTo(contrastDB);

			char slot=DisplayTextWindow(DISP_WND_BOTTOM,LoadText(TEXT_CHOS_LOAD),FALSE);

			if(slot>=1 && slot<=3)
			{
				if(!LoadGame(slot)) DisplayTextWindow(DISP_WND_TOP,LoadText(256),FALSE);
				else rtnChoice=CHOICE_PLAY;
			}
		}

		if(rtnChoice==CHOICE_NEW)
		{
			InitGame();
			ContrastTo(contrastDB);
			rtnChoice=CHOICE_PLAY;
		}

		if(rtnChoice==CHOICE_PLAY)
		{			
			Play();
			if(hero.m_life<=0) GameOver();
		}

		if(rtnChoice==CHOICE_QUIT)	QuitGame();

		//load start menu
		LoadBitmap(DEFAULT_INTRO);	
		LoadMap(0);
		rtnChoice=StartMenu();

	} while(rtnChoice!=CHOICE_QUIT);
	
	QuitGame();
}

void QuitGame()
{
	//clear screen and memory
	ClearScreen(SEG0);
	ClearScreen(SEG1);
	
	/*delete []curMap.tabSpr;
	delete []curMap.tabProp;*/


	//show credits
	ContrastTo(contrastDB);
	DisplayTextWindow(DISP_WND_TOP,LoadText(TEXT_FOR_PEOPLE),FALSE);
	
	//clear screen and memory
	ClearScreen(SEG0);
	ClearScreen(SEG1);

	//default video mode
	SetMode(MODE_C3);
	ContrastTo(contrastC3);
	
	//save config
	if(!SaveConfig()) 
	{
		printf(DEFAULT_MEM_ERROR);
		while(!Touche(MKS_ESC));
	}

	create_mem_zone(DEFAULT_MEM_ZONE,DEFAULT_MEM_MAP,SIZE_SPR_MAP+SIZE_ZONE,DELETE_ZONE);
	create_mem_zone(DEFAULT_MEM_ZONE,DEFAULT_MEM_ITEM,SIZE_SPR_ITEM+SIZE_ZONE,DELETE_ZONE);
	create_mem_zone(DEFAULT_MEM_ZONE,DEFAULT_MEM_FONT,SIZE_SPR_FONT+SIZE_ZONE,DELETE_ZONE);
	create_mem_zone(DEFAULT_MEM_ZONE,DEFAULT_MEM_BITMAP,SIZE_BITMAP+SIZE_ZONE,DELETE_ZONE);
	
	//init linked lists
	scenery.DeleteAll();
	
	//Enable the key buffer
	SetEnableKeyBuf();
	
	Exit();
}

void Play()
{
	uchar timeS;
	uchar FPS;
	uchar compteFPS;
	
	while(!Touche(MKS_DEL) && hero.m_life>0)
	{
		hero.m_aniLen=ANI_NONE;
		hero.m_haveMoved=FALSE;
		
		//test keys
		if(Touche(MKS_DOWN)) hero.MoveTo(DIR_BOTTOM);
		if(Touche(MKS_LEFT)) hero.MoveTo(DIR_LEFT);
		if(Touche(MKS_UP)) hero.MoveTo(DIR_TOP);
		if(Touche(MKS_RIGHT)) hero.MoveTo(DIR_RIGHT);
		
		//update hero weapons
		if(hero.m_weapon1!=NULL)
		{
			if(Touche(KEY_WEAPON_1)) hero.m_weapon1->StartAction();
			hero.m_weapon1->Action();
		}
		/*if(hero.m_weapon2!=NULL)
		{
			if(Touche(KEY_WEAPON_2)) hero.m_weapon2->StartAction();
			hero.m_weapon2->Action();
		}*/

		//test collision, move enemies...
		GlobalUpdate();

		//draw all elements	
		DrawScene();

		uchar tmpS=inportb(0x1D);	//comptage des fps
		
		if(timeS!=tmpS)
		{
			FPS=compteFPS;
			compteFPS=0;
			timeS=tmpS;
		}
		compteFPS++;

		SwapPage();
		
		if(Touche(MKS_F1))	DisplayTextWindow(DISP_WND_TOP,LoadText(1));
		
		if(Touche(MKS_F2))
		{
			sprintf(curText,"%i FPS.\n%i free\n%i enemies loaded\n%i objects loaded", FPS, coreleft(), enemies.CountSprite(), items.CountSprite());
			DisplayTextWindow(DISP_WND_TOP,curText);
		}
		if(Touche(KEY_INVENTORY)) Inventory();

		//if(Touche(MKS_XOT)) SetMapItemIsScan(GetMapID(),FALSE);
	}
}


void GameOver()
{
	ClearPageWith(SEG3, 0xFFFF);

	LoadBitmap(DEFAULT_GAMEOVER);
	DrawM21Bitmap(SEG1);

	switch(DisplayTextWindow(DISP_WND_BOTTOM,LoadText(TEXT_GAME_OVER),FALSE))
	{
	case CHOOSE_1:
		QuitGame();
	case CHOOSE_2:
		break;
	}
}

void Intro()
{
	mapPosX=0;
	mapPosY=0;
	char nShad=-16;
	uchar i;
	CNode* curNode;
	
	//init variables
	globalTimer=0;
	uchar state=STATE_INTRO_FADE;
	uchar upContrast=0;	
	
	//Intro
	ContrastTo(0);
	while(!Touche(MKS_ESC) && state!=STATE_MENU_NEW)
	{
		//general for all states
		ClearScreen(SEG1);
		
		if(globalTimer%3==0)
		{
			mapPosX+=incx;
			mapPosY+=incy;
		}
		
		if(mapPosX>=posMapMaxPxX || mapPosX<=0) incx=-incx;
		if(mapPosY>=posMapMaxPxY || mapPosY<=0) incy=-incy;
		
		
		//states
		switch(state)
		{
		case STATE_INTRO_FADE:
			DrawM21Bitmap(SEG1);
			DrawValueLayer(0x0000, 64, SEG1);
			DrawValueLayer(0x0000, 64, SEG3);
			DrawMap(mapPosX,mapPosY);
			for(curNode=scenery.m_head; curNode!=NULL; curNode=curNode->m_next)
			{
				//sprite is always enable, or doesn't exist...
				curNode->m_sprite->Draw();
			}
			if(globalTimer%2==0 && upContrast<contrastDB)
			{
				upContrast++;
				ContrastP();
			}
			if(globalTimer==64) state=STATE_INTRO_SHADOW;
			break;
			
		case STATE_INTRO_SHADOW:
			DrawM21Bitmap(SEG1);
			DrawValueLayer(0x0000, 64, SEG1);
			DrawValueLayer(0x0000, 64, SEG3);
			DrawMap(mapPosX,mapPosY);
			for(curNode=scenery.m_head; curNode!=NULL; curNode=curNode->m_next)
			{
				//sprite is always enable, or doesn't exist...
				curNode->m_sprite->Draw();
			}
			if(nShad>0) DrawValueLayer(0xFFFF, nShad, SEG3);
			for( i=0; i<16; i++) DrawM21Spr_C(&memBitmap, i*8, 64-(nShad+16), i+224,SEG1);
			for( i=0; i<16; i++) DrawM21Spr_C(&memBitmap, i*8, 64-(nShad+8), i+240,SEG1);
			nShad++;
			if(nShad==64) state=STATE_MENU_NEW;
			break;
		}
		
		//general for all states
		SwapPage();
	}
	if(upContrast<contrastDB) ContrastTo(contrastDB);
}

uchar StartMenu()
{
	uchar state=STATE_MENU_NEW;
	uchar choice=CHOICE_NEW;
	ushort i;
	uchar timerWait;

	//init linked lists
	enemies.DeleteAll();
	items.DeleteAll();
	hero.m_listItems.DeleteAll();
	hero.m_listWeapons.DeleteAll();
	hero.m_weapon1=NULL;

	//start menu begin here
	while(!Touche(MKS_EXE))
	{
		//general for all states
		ClearScreen(SEG1);

		if(Touche(MKS_F6)) QuitGame();
		
		if(globalTimer%3==0)
		{
			mapPosX+=incx;
			mapPosY+=incy;
			timerWait++;
		}
		
		if(mapPosX>=posMapMaxPxX) 
		{
			mapPosX=posMapMaxPxX;
			incx=-1;
		}
		if(mapPosY>=posMapMaxPxY) 
		{
			mapPosY=posMapMaxPxY;
			incy=-1;
		}
		if(mapPosX<=0) 
		{
			mapPosX=0;
			incx=1;
		}
		if(mapPosY<=0) 
		{
			mapPosY=0;
			incy=1;
		}
		
		DrawMap(mapPosX,mapPosY);
		for(CNode* curNode=scenery.m_head; curNode!=NULL; curNode=curNode->m_next)
		{
			//sprite is always enable, or doesn't exist...
			curNode->m_sprite->Draw();
		}
		DrawValueLayer(0xFFFF, 64, SEG3);
		DrawM21Bitmap(SEG1);

		DrawM21SprW(&memBitmap, 112, 8, SPR_ROTATE+timerWait%2, SEG1);
		DrawM21SprW(&memBitmap, 120, 8, SPR_ROTATE+2+timerWait%2, SEG1);

		//states
		switch(state)
		{
		case STATE_MENU_NEW:
			for(i=0; i<4; i++) DrawM21SprW(&memBitmap, i*8+96, 48, SPR_MENU_NEW+i, SEG1);
			if(Touche(MKS_UP))
			{
				choice=CHOICE_QUIT;
				state=STATE_MENU_NEW_F;
				timerWait=0;
			}
			if(Touche(MKS_DOWN))
			{
				choice=CHOICE_LOAD;
				state=STATE_MENU_NEW_F;
				timerWait=0;
			}
			break;
		case STATE_MENU_LOAD:
			for(i=0; i<4; i++) DrawM21SprW(&memBitmap, i*8+96, 48, SPR_MENU_LOAD+i, SEG1);
			if(Touche(MKS_UP))
			{
				choice=CHOICE_NEW;
				state=STATE_MENU_LOAD_F;
				timerWait=0;
			}
			if(Touche(MKS_DOWN))
			{
				choice=CHOICE_QUIT;
				state=STATE_MENU_LOAD_F;
				timerWait=0;
			}
			break;
		case STATE_MENU_QUIT:
			for(i=0; i<4; i++) DrawM21SprW(&memBitmap, i*8+96, 48, SPR_MENU_QUIT+i, SEG1);
			if(Touche(MKS_UP))
			{
				choice=CHOICE_LOAD;
				state=STATE_MENU_QUIT_F;
				timerWait=0;
			}
			if(Touche(MKS_DOWN))
			{
				choice=CHOICE_NEW;
				state=STATE_MENU_QUIT_F;
				timerWait=0;
			}
			break;
		case STATE_MENU_NEW_F:
			for(i=0; i<4; i++) DrawM21SprW(&memBitmap, i*8+96, 48, SPR_MENU_NEW+timerWait*4+i, SEG1);
			if(timerWait==3)
			{
				if(choice==CHOICE_LOAD) state=STATE_MENU_LOAD;
				else state=STATE_MENU_QUIT;
			}
			break;
		case STATE_MENU_LOAD_F:
			for(i=0; i<4; i++) DrawM21SprW(&memBitmap, i*8+96, 48, SPR_MENU_LOAD+timerWait*4+i, SEG1);
			if(timerWait==3)
			{
				if(choice==CHOICE_QUIT) state=STATE_MENU_QUIT;
				else state=STATE_MENU_NEW;
			}
			break;
		case STATE_MENU_QUIT_F:
			for(i=0; i<4; i++) DrawM21SprW(&memBitmap, i*8+96, 48, SPR_MENU_QUIT+timerWait*4+i, SEG1);
			if(timerWait==3)
			{
				if(choice==CHOICE_NEW) state=STATE_MENU_NEW;
				else state=STATE_MENU_LOAD;
			}
			break;
		}
		
		//general for all states
		SwapPage();
	}

	for(i=0; i<20; i++)
	{
		ContrastM();
		Wait(10);
	}

	ClearScreen(SEG0);
	ClearScreen(SEG1);

	return choice;
}

void Inventory()
{
	char i;
	char *keepBmpFile=curBmpFile;
	CBaseSpr* selObject;

	LoadBitmap(DEFAULT_INVENTORY);
	
	while(!Touche(MKS_ESC))
	{
		//commun for 2 inventories
		ClearScreen(SEG1);
		DrawMap(mapPosX,mapPosY);
		for(CNode* curNode=scenery.m_head; curNode!=NULL; curNode=curNode->m_next)
		{
			//sprite is always enable, or doesn't exist...
			curNode->m_sprite->Draw();
		}
		ClearPageWith(SEG3, 0xFFFF);

		DrawM21Bitmap(SEG1);
		DrawStatusBar();
		

		//test navigation
		if(Touche(MKS_DOWN)) 
		{
			lastInvSel+=4;
			Wait(100);
		}
		if(Touche(MKS_LEFT)) 
		{
			lastInvSel--;
			Wait(100);
		}
		if(Touche(MKS_UP))
		{
			lastInvSel-=4;
			Wait(100);
		}
		if(Touche(MKS_RIGHT))
		{
			lastInvSel++;
			Wait(100);
		}

		if(Touche(MKS_F2)) 
		{
			lastInventory=INV_ITEMS;
			Wait(100);
		}	
		
		if(Touche(MKS_F1)) 
		{
			lastInventory=INV_WEAPONS;
			Wait(100);
		}
		
		switch(lastInventory)
		{
		case INV_WEAPONS:
			if(lastInvSel>=hero.m_listWeapons.CountSprite()) lastInvSel=hero.m_listWeapons.CountSprite()-1;
			
			//draw tab
			DrawM21SprW(&memBitmap, 0, 56, INV_SPR_WEAPONS, SEG1);
			DrawM21SprW(&memBitmap, 8, 56, INV_SPR_WEAPONS+1, SEG1);

			if(lastInvSel==-1) break;
			
			//sel current weapon
			selObject=hero.m_listWeapons.GetSprite(lastInvSel);
			
			//draw sel weapon
			DrawM21SprW(&memSprItem,56,8,((CWeapon*)selObject)->m_sprID[DIR_RIGHT],SEG1);
			
			//draw other weapons
			for(i=0; i<hero.m_listWeapons.CountSprite(); i++)
			{
				DrawM21SprW(&memSprItem, 8+(i%5)*8,8+(i/5)*8,((CWeapon*)hero.m_listWeapons.GetSprite(i))->m_sprID[DIR_RIGHT],SEG1);
			}
			
			//break if weapon is active
			if(hero.m_weapon1!=NULL && hero.m_weapon1->IsEnable()) break;


			//pick up the weapon
			if(Touche(MKS_CTRL)) 
			{
				hero.m_weapon1=(CWeapon*) selObject;
			}

			//pick up weapon and exit
			if(Touche(KEY_WEAPON_1))
			{
				hero.m_weapon1=(CWeapon*) selObject;
				Wait(1000);
				LoadBitmap(keepBmpFile);
				return;
			}
			
			break;
		case INV_ITEMS:
			if(lastInvSel>=hero.m_listItems.CountSprite()) lastInvSel=hero.m_listItems.CountSprite()-1;

			//draw tab
			DrawM21SprW(&memBitmap, 16, 56, INV_SPR_ITEMS, SEG1);
			DrawM21SprW(&memBitmap, 24, 56, INV_SPR_ITEMS+1, SEG1);

			if(lastInvSel==-1) break;
			
			//sel current item
			selObject=hero.m_listItems.GetSprite(lastInvSel);
			
			//draw sel item
			DrawM21SprW(&memSprItem,56,8,GetAniSprItem(((CItem*)selObject)->m_sprID, ((CItem*)selObject)->m_aniLen, ((CItem*)selObject)->m_aniSpeed),SEG1);
			
			//draw other items
			for(i=0; i<hero.m_listItems.CountSprite(); i++)
			{
				DrawM21SprW(&memSprItem, 8+(i%5)*8,8+(i/5)*8,GetAniSprItem(((CItem*)hero.m_listItems.GetSprite(i))->m_sprID, ((CItem*)hero.m_listItems.GetSprite(i))->m_aniLen, ((CItem*)hero.m_listItems.GetSprite(i))->m_aniSpeed),SEG1);
			}
			
			//use the item
			if(Touche(MKS_CTRL)) 
			{
				BOOL wantExit=((CItem*)selObject)->Use();
				WipeInvHero();
				Wait(200);
				if(wantExit) 
				{
					LoadBitmap(keepBmpFile);
					return;
				}
			}
			break;
		}

		if(lastInvSel>=0 && lastInventory!=INV_NONE)
		{
			DrawM21SprW(&memBitmap, 8+(lastInvSel%5)*8, 8+(lastInvSel/5)*8, INV_SPR_SEL+(globalTimer/4)%2, SEG1);
			
			//draw help
			if(Touche(MKS_F3)) DisplayTextWindow(DISP_WND_BOTTOM,LoadText(TEXT_HELP_OBJECT+selObject->m_objectID),FALSE);
				
			//draw extra info
			sprintf(curText,"%0003u", ((CSprite*)selObject)->m_extra);
			DrawText(90,8,curText);
		}

		//update screen
		SwapPage();

		if(lastInventory==INV_NONE)
		{
			DisplayTextWindow(DISP_WND_TOP,LoadText(TEXT_HELP_INVENTORY),FALSE);
			lastInventory=INV_WEAPONS;
		}
	}

	UpdateItem();
	LoadBitmap(keepBmpFile);
}
