/**
* @package Types
*/
/****************************************************
 *	Part of Shadow World project					*
 *													*
 *	Original name : Type2.h							*
 *													*
 *	Type, struct declarations for other files.		*
 *													*
 *													*
 *	May 2003, LeadFiner								*
 *	LeadFiner@wanadoo.fr							*
 ****************************************************/

//#include <conio.h> //for test only
#include <stdio.h>
#include <STDLIB.H>

#if !defined TYPES
#define TYPES

#include "DefObj.h"

/********************************************************************************
 *																				*
 *						Defines													*
 *																				*
 ********************************************************************************/

#if !defined WIN32
	//for BOOL
	#define TRUE	-1
	#define FALSE	0

	#pragma option -r  //register variable can be used
	#pragma option -Z  //register re-use variable optimization
	#pragma option -v- //De-activate debugging and expand inline functions
	#pragma option -y- //No line number in executable !
	#pragma option -N- //no stack overflow is made.
	#pragma option -O  //delete unrechable code in exe.
#endif



//Shadow World Version
#define SW_VERSION		203



//for video
#define SEG0	0x1A20
#define SEG1	0x1A60
#define SEG2	0x1AA0
#define SEG3	0x1AE0
#define REF_CONTRAST	18

#if !defined WIN32
	//for pointers
	#define NULL	0
#endif

//define max
#define MAP_SIZE_MAX	1024
#define MAP_ANI_MAX		16
#define MAX_MAP			256
#define MAX_CAR			512

//for maps
#define MAP_MAX_LOADED	MAX_MAP/8		//for tabMapLoaded[]

//for items
#define DEFAULT_SPR_SHADOW	244
#define DEFAULT_SPR_EXPLODE	128
#define DEFAULT_SPR_CHRONO	245
#define DEFAULT_ANI_SPEED	5
#define ANI_NONE			0


//init screen
#define DIM_SCR_X			14
#define DIM_SCR_Y			8
#define DIM_SCR_PX_X		DIM_SCR_X*8
#define DIM_SCR_PX_Y		DIM_SCR_Y*8
#define POS_SCR_C_PX_X		DIM_SCR_PX_X/2
#define POS_SCR_C_PX_Y		DIM_SCR_PX_Y/2

//for sprites
#define TIME_ENABLE			2048				//approx 1min for 30FPS
#define TIME_NONE			TIME_ENABLE+5		//disable timer
#define TIME_4VAR_OFFSET	TIME_ENABLE+6		//use m_timerEnable as variable


//for files
#define DEFAULT_SPR_MAP		"SPRITE.SP2"
#define DEFAULT_SPR_ITEM	"SPRITE.SP3"
#define DEFAULT_SPR_FONT	"FONT.SP3"
#define DEFAULT_TEXT		"TEXT.GTX"
#define DEFAULT_INTRO		"INTRO.BIT"
#define DEFAULT_GAMEOVER	"GAMEOVER.BIT"
#define DEFAULT_INVENTORY	"HEROINV.BIT"
#define DEFAULT_WORLD		"WORLD.WLD"
#define SW_EXE				"SW.EXE"
#define DEFAULT_MEM_ERROR	"\nCould not allocate\ntemporary memory...\nYou must free at\nleast 20Ko of RAM forplaying the game."
#define DEFAULT_LOADING		"\nPlease wait\nwhile loading..."

//for bmp files FX
#define BITMAP_HALO			"HALO.BIT"


//for memory
#define SIZE_ZONE			14			//minimum for a memory zone file...
#define SIZE_SPR_ITEM		256*8*3		//256 sprites, 8 lines, 3 layers
#define SIZE_SPR_MAP		256*8*2		//256 sprites, 8 lines, 2 layers
#define SIZE_SPR_FONT		112*8*3		//96 sprites, 8 lines, 3 layers
#define SIZE_BITMAP			256*8*3		//256 sprites, 8 lines, 3 layers
#define SIZE_TAB_MAX		256*8*3		//size max of memzone tab
#define SIZE_SAVE			2			//minimum for save file...
#define SIZE_CFG			4			//minimum for save file...

#define DEFAULT_MEM_MAP		"SW_MAP"
#define DEFAULT_MEM_ITEM	"SW_ITEM"
#define DEFAULT_MEM_FONT	"SW_FONT"
#define DEFAULT_MEM_BITMAP	"SW_BMP"
#define DEFAULT_MEM_SAVE1	"SW_SAVE1"
#define DEFAULT_MEM_SAVE2	"SW_SAVE2"
#define DEFAULT_MEM_SAVE3	"SW_SAVE3"
#define DEFAULT_MEM_CFG		"SW_CFG"

#define DEFAULT_MEM_ZONE	12


//for keys
#define KEY_WEAPON_1		MKS_SHIFT
//#define KEY_WEAPON_2		MKS_ALPHA
#define KEY_INVENTORY		MKS_VARS
#define KEY_ACTION			MKS_CTRL


//for texts
#define TEXT_INTRO			0
#define TEXT_FOR_PEOPLE		2
#define TEXT_DONT_GO		64

#define TEXT_NOT_LOADED		256
#define TEXT_DO_SAVE		257
#define TEXT_OK_SAVED		258
#define TEXT_NOT_SAVED		259
#define TEXT_ITEM_NOT_USE	260
#define TEXT_GAME_OVER		261
#define TEXT_NOT_SHINES		262
#define TEXT_THANK_YOU		263
#define TEXT_BUYER_1		264
#define TEXT_HELP_INVENTORY	265
#define TEXT_CHOS_SAVE		266
#define TEXT_CHOS_LOAD		267
#define TEXT_MEGA_TELE2		268
#define TEXT_NOT_USE_IN_MAP	269


#define TEXT_HELP_OBJECT	300



/********************************************************************************
 *																				*
 *						Typedef Declarations									*
 *																				*
 ********************************************************************************/

typedef unsigned char uchar;
typedef unsigned short ushort;
typedef unsigned long ulong;

#if !defined WIN32
	typedef char BOOL;
	typedef unsigned long DWORD;
#endif
/********************************************************************************
 *																				*
 *						Struct Declarations										*
 *																				*
 ********************************************************************************/

struct StructRect_c
{
	char bottom;
	char left;
	char top;
	char right;
};

struct StructAni
{
	unsigned char Start;
	unsigned char End;
	unsigned char Speed;
};

struct StructJump
{
	unsigned char MapID;
	unsigned short CaseNbr;
};

/*#if !defined WIN32
	struct StructMap 
	{
		unsigned char dimX;
		unsigned char dimY;
		unsigned char *tabSpr;
		unsigned char *tabProp;
		StructJump tabJump[MAP_JUMP_MAX];
		StructAni tabAni[MAP_ANI_MAX];
		char maxAni;
		unsigned char tabText[MAP_TEXT_MAX];

		uchar mapID;

	};
#endif*/

//#if defined WIN32
	struct StructMap 
	{
		unsigned char tabSpr[MAP_SIZE_MAX];
		unsigned char tabProp[MAP_SIZE_MAX];
		StructJump tabJump[MAP_JUMP_MAX];
		StructAni tabAni[MAP_ANI_MAX];
		char maxAni;
		unsigned char tabText[MAP_TEXT_MAX];

		unsigned char dimX;
		unsigned char dimY;
		uchar mapID;
	};
//#endif
/********************************************************************************
 *																				*
 *								Extra includes									*
 *																				*
 ********************************************************************************/

#endif